// 230330

`timescale 1ns/1ps	// time-unit, precision
`default_nettype none

module MatrixKeyboard_tb;

	parameter CLOCK_FREQ_HZ	= 1_000_000;
	parameter HALF_PERIOD_NS = 1_000_000_000 / (2 * CLOCK_FREQ_HZ);

	// Clock generator
	reg	Clock = 1'b1;
	always begin
		#HALF_PERIOD_NS;
		Clock = !Clock;
	end
	
	// Signals
	reg				Reset	= 1'b1;
	reg		[3:0]	Rows	= 4'd0;
	wire	[7:0]	Cathodes;
	wire	[7:0]	Segments;
	wire			SwitchCathode;
	wire			KeyStrobe;
	wire			KeyPressed;
	wire	[4:0]	KeyCode;
	
	// Simulate keypad rows
	always @(Cathodes) begin
		if(DUT.State == DUT.ANALYZE)
			$error("Cathodes changed during analysis");
		
		//if($realtime < 400*1000)
		if(!KeyPressed)
			case(Cathodes)
				8'b00000001:	Rows = 4'b0000;
				8'b00000010:	Rows = 4'b0000;
				8'b00000100:	Rows = 4'b0100;
				8'b00001000:	Rows = 4'b0000;
				8'b00010000:	Rows = 4'b0000;
				8'b00100000:	Rows = 4'b0000;
				8'b01000000:	Rows = 4'b0000;
				8'b10000000:	Rows = 4'b0000;
				default:		Rows = 4'bxxxx;
			endcase
		else
			Rows = 4'b0000;
	end
	
	// Test sequence
	initial begin
		$timeformat(-6, 0, "us", 10);
		$display("===== START =====");
		$display("FREQUENCY_HZ = %9d", CLOCK_FREQ_HZ);
		$display("DELAY        = %9d", DisplayMultiplex0.StrobeGenerator0.DELAY);
		$display("WIDTH        = %9d", DisplayMultiplex0.StrobeGenerator0.WIDTH);
		
		#5 Reset = 1'b0; 
		#5 Reset = 1'b1;
		
		$display("      Time  Cathode  Rows Pressed Code");
		repeat(30) begin
			@(posedge SwitchCathode);
			$display("%t        %d  %b       %b   %d", 
				$realtime, 
				DisplayMultiplex0.Selector,
				Rows,
				KeyPressed,
				KeyCode
			);
		end
		
		#100;
		
		$display("===== END =====");
		#1 $finish;
	end
	
	// Instantiate matrix display driver
	DisplayMultiplex #(
		.FREQUENCY_HZ(CLOCK_FREQ_HZ),
		.SWITCH_PERIOD_US(50),
		.DIGITS(8)
	) DisplayMultiplex0(
		.Clock(Clock),
		.Reset(Reset),
		.Data(32'h01234567),
		.DecimalPoints(8'b00000001),
		.Cathodes(Cathodes),
		.Segments(Segments),
		.SwitchCathode(SwitchCathode)
	);
	
	// Instantiate device under test
	MatrixKeyboard DUT(
		.Clock(Clock),
		.Reset(Reset),
		.SwitchCathode(SwitchCathode),
		.Rows(Rows),
		.Cathodes(Cathodes),
		.KeyStrobe(KeyStrobe),
		.KeyPressed(KeyPressed),
		.KeyCode(KeyCode)
	);
	
	// Variable dump	
	initial begin
		$dumpfile("matrix_keyboard.vcd");
		$dumpvars(0, MatrixKeyboard_tb);
	end

endmodule
`default_nettype wire
